BEGIN TRANSACTION;
CREATE TABLE Audit (
    Num         INTEGER     PRIMARY KEY AUTOINCREMENT NOT NULL,
    File        TEXT        NOT NULL,
    Action      TEXT        NOT NULL,
    Stamp       TEXT        NOT NULL
);
DELETE FROM sqlite_sequence;
INSERT INTO "sqlite_sequence" VALUES('Audit',9669);
CREATE TABLE Playlist (
    Name        TEXT        PRIMARY KEY NOT NULL
);
CREATE TABLE PlaylistSong (
    Name        TEXT        NOT NULL,
    File        TEXT        NOT NULL,
    CONSTRAINT pk_PlaylistSong         PRIMARY KEY (Name,File),
    CONSTRAINT fkPlaylist_PlaylistSong FOREIGN KEY (Name) REFERENCES Playlist (Name) ON UPDATE CASCADE ON DELETE CASCADE, 
    CONSTRAINT fkSong_PlaylistSong     FOREIGN KEY (File) REFERENCES Song (File) ON UPDATE CASCADE ON DELETE CASCADE    
);
CREATE TABLE 'Song' (
    File        TEXT    NOT NULL,
    Artist      TEXT,
    Album       TEXT,
    Title       TEXT    NOT NULL,
    Genre       TEXT,
    Time        NUMBER  NOT NULL,
    Track       TEXT,
    Date        NUMBER,
    Rating      NUMBER,
    Art         TEXT,
    CONSTRAINT pkHost_Song PRIMARY KEY (File)
);
ANALYZE sqlite_master;
INSERT INTO "sqlite_stat1" VALUES('statistic','sqlite_autoindex_statistic_1','4 1');
CREATE TABLE Statistic (
    Key         TEXT    NOT NULL,
    Value       TEXT    NOT NULL DEFAULT "0",
    CONSTRAINT pkHost_Statistic PRIMARY KEY (Key)
);
INSERT INTO "Statistic" VALUES('artists','0');
INSERT INTO "Statistic" VALUES('songs','0');
INSERT INTO "Statistic" VALUES('db_playtime','0');
INSERT INTO "Statistic" VALUES('db_update','0');
INSERT INTO "Statistic" VALUES('program_version','1.0.1');
CREATE INDEX idxHost_Audits ON Audit (File, Action);
CREATE VIEW Decades AS
	SELECT 
	    File AS File,
		CASE
			WHEN (Song.Date > 1000 AND Song.Date < 3000) THEN Song.Date / 10 * 10
			ELSE NULL
		END AS Decade
 	FROM Song
 	ORDER BY Decade;
CREATE VIEW favSongs AS
	SELECT 
		File AS File,
		COUNT(*) AS Instance
	FROM Audit
	WHERE UPPER(Action) IN ('ADDED','PLAY')
	GROUP BY File
	ORDER BY Instance DESC
	LIMIT 150;
CREATE VIEW favAlbum AS
	SELECT 
		so.Album AS Album,
		COUNT(*) AS Instance
	  FROM Song so
 LEFT JOIN  Audit au ON so.File = au.File
	 WHERE UPPER(au.Action) IN ('ADDED','PLAY')
  GROUP BY Album
  ORDER BY Instance DESC
	 LIMIT 10;
CREATE VIEW favArtist AS
	SELECT 
		so.Artist AS Artist,
		COUNT(*) AS Instance
	  FROM Song so
 LEFT JOIN  Audit au ON so.File = au.File
	 WHERE UPPER(au.Action) IN ('ADDED','PLAY')
  GROUP BY Artist
  ORDER BY Instance DESC
	 LIMIT 10;
CREATE VIEW favGenre AS
	SELECT 
		so.Genre AS Genre,
		COUNT(*) AS Instance
	  FROM Song so
 LEFT JOIN  Audit au ON so.File = au.File
	 WHERE UPPER(au.Action) IN ('ADDED','PLAY')
  GROUP BY Genre
  ORDER BY Instance DESC
	 LIMIT 10;
CREATE VIEW favRating AS
    SELECT 
        File AS File,
        Rating AS Rating
    FROM Song
    WHERE Rating IS NOT NULL 
           AND Rating > 0
    ORDER BY Rating DESC
    LIMIT 150;
COMMIT;
