/**
 * Copyright (c) 2011, Martin Foster <martin@ethereal-realms.org>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 *     * Redistributions of source code must retain the above copyright notice, 
 *       this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright 
 *       notice, this list of conditions and the following disclaimer in the 
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Ethereal-Realms.org nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.lyra.Block;

import org.lyra.AbstractBlock;
import org.lyra.Widgets.InfiniteProgressPanel;

/**
 * The org.lyra.Widgets.InfiniteProgressPanel class is the preferred method
 * for blocking the user interface.  However it can lead to slow downs due to
 * the graphics intensive method.
 * <p>
 * @author Martin Foster
 * @version 1.0.1, %Revision, 99%, %LastChangedDate, 17/03/11 8:17PM%
 * @see org.lyra.Widgets.InfiniteProgressPanel
 */
public class InfiniteProgressImpl extends AbstractBlock {

	// Class Objects
	private InfiniteProgressPanel Block;
	
	
	/**
	 * Constructor for the class.  Provided with an instance of the 
	 * <i>org.lyra.Widgets.InfiniteProgressPanel</i> will interface an
	 * application class with a proper interface.
	 * <p>
	 * @param block Instance of InfiniteProgressPanel.
	 */
	public InfiniteProgressImpl(InfiniteProgressPanel block) {
		// Pass up
		this.Block = block;
	}
	// End of Constructor Method
	
	
	/* (non-Javadoc)
	 * @see org.lyra.AbstractBlock#startBlock()
	 */
	public void startBlock() {
		
		// Direct pass-through
		Block.start();
	}
	// End of startBlock method

	/* (non-Javadoc)
	 * @see org.lyra.AbstractBlock#stopBlock()
	 */
	@Override
	public void stopBlock() {
		
		// Direct pass-through
		Block.stop();
	}
	// End of stopBlock method


	/* (non-Javadoc)
	 * @see org.lyra.AbstractBlock#setMessage(java.lang.String)
	 */
	@Override
	public void setMessage(String msg) {

		// Direct pass-through
		Block.setText(msg);
	}
	// End of setMessage Method

	
	/* (non-Javadoc)
	 * @see org.lyra.AbstractBlock#getMessage()
	 */
	@Override
	public String getMessage() {

		// Direct pass-through
		return Block.getText();
	}
	// End of getMessage Method
}
